#!/usr/bin/env python
# -*- coding: utf-8 -*-

import subprocess
import time
import os

# ###########################################################
# pinger.py, D.A. 10.11.2013
#
# Dieses Script prüft den Online-Status der unter "hosts2ping"
# angegebenen Hosts. Statt ping wird fping verwendet.
# Änderungen am Online-Status werden zusammen mit dem jew.
# Timestamp in die Datei gemäß "logfile" geschrieben.
# ###########################################################

logfile="/home/dede/log_pinger.txt"
hosts2ping=[ "powerline", "c2q", "i5", "t22", "rp1", "rp2", "bb", "t430", "tobi", "mutti", "Wii", "tablet", "buffalo", "switch", "gs2", "ipod", "sony-handy" ]


onlineHosts=[]
timeformat="%Y%m%d-%H%M%S"

FNULL=open(os.devnull, 'w')   # DEVNULL gibts erst ab Python 3.x


# ###########################################################
# Ping alle Hosts aus "hosts2ping" an und liefert die
# Hostnamen als Liste zurück, die online sind.
#
# Exit status of fping is 
# 0 if all the hosts are reachable, 
# 1 if some hosts were unreachable, 
# 2 if any IP addresses were not found, 
# 3 for invalid command line arguments, and 
# 4 for a system call failure.
def pinging():
  cmd_l=["fping", "-a"] + hosts2ping
  process=subprocess.Popen(cmd_l, shell=False, stdout=subprocess.PIPE, stderr=FNULL)
  rc=process.communicate()[0]

  ret_lst=[]
  if process.returncode<3:
    zeilenweise=rc.split("\n")
    for ln in zeilenweise:
      if ln.strip()!="":
        ret_lst.append(ln.strip())
  else:
    print "rc=", process.returncode
  return(ret_lst)


# ###########################################################
# Wandelt eine Liste wie etwa   ['c2q', 't22', 'switch']
# nach                          c2q, t22, switch
def list2string(lst):
  s=""
  for h in lst:
    s+=", "+h
  return(s[2:])


# ###########################################################
# Schreibt "txt" in die Datei "logfile".
def logStatus(txt):
  fobj=open(logfile, "a")
  fobj.write(txt+"\n")
  fobj.close()



# ###########################################################
# #################                         #################
# ################       Hauptprogramm       ################
# #################                         #################
# ###########################################################

z=time.strftime(timeformat)
onlineHosts=pinging()
logStatus(z + " restart " + "-"*50)
logStatus(z + " online  " + list2string(onlineHosts))

while True:
  time.sleep(60)
  z=time.strftime(timeformat)
  hosts=pinging()

  onl=[]
  for h in hosts:         # neu pingende Hosts finden
    if h not in onlineHosts:
      onl.append(h)
      onlineHosts.append(h)
  if onl!=[]:
    logStatus(z + " online  " + list2string(onl))

  offl=[]
  for h in onlineHosts:   # nicht mehr pingende Hosts finden
    if h not in hosts:
      offl.append(h)
      onlineHosts.remove(h)
  if offl!=[]:
    logStatus(z + " offline " + list2string(offl))

